/***********************************************************************

Vorosketch User Distances Demo by Herman Haverkort, 17 September 2022

************************************************************************

Copyright 2022 Herman Johannes Haverkort

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

  http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

This file is designed to be used with Vorosketch v0.24 BETA; it might 
work with later versions too. New versions may be available from 
http://herman.haverkort.net.


***********************************************************************/


#define MAKEUSERDISTANCE \
if (dname == "mydistance") return new MyDistance();\
if (dname == "wave") return new WaveDistance();


class MyDistance: public PointDistance
{
    public:
  virtual const string description() const
  {
    return "mydistance";
  }

  virtual DOUBLE operator()(Point& p, const Site& s) const
  {
    return (p - s.points[0]).r();
  }
};


class WaveDistance: public SetDistance
{
    public:
  virtual const string description() const
  {
    return "wave";
  }

  virtual DOUBLE operator()(Point& p, const Site& s) const
  {
    DOUBLE d = EuclideanDistance()(p, s);
    return(1.0 + (sin(d * 10 - PI * 0.5) - 1.0) * pow(0.1, d) * 0.5);
  }
};
